@@ -325,7 +325,7 @@ public:
 		inline pkgCache::DepIterator RevDependsList() const { return getType().RevDependsList(); }
 		inline pkgCache::PrvIterator ProvidesList() const { return getType().ProvidesList(); }
 		inline pkgCache::PkgIterator::OkState State() const { return getType().State(); }
-		inline const char *CandVersion() const { return getType().CandVersion(); }
+		APT_DEPRECATED_MSG("This method does not respect apt_preferences! Use pkgDepCache::GetCandidateVersion(Pkg)") inline const char *CandVersion() const { return getType().CandVersion(); }
 		inline const char *CurVersion() const { return getType().CurVersion(); }
 		inline pkgCache *Cache() const { return getType().Cache(); }
 		inline unsigned long Index() const {return getType().Index();}
@@ -465,7 +465,7 @@ APT_IGNORE_DEPRECATED_POP
 	/** \brief returns all packages in the cache who belong to the given task
 
 	    A simple helper responsible for search for all members of a task
-	    in the cache. Optional it prints a a notice about the
+	    in the cache. Optional it prints a notice about the
 	    packages chosen cause of the given task.
 	    \param Cache the packages are in
 	    \param pattern name of the task
@@ -483,7 +483,7 @@ APT_IGNORE_DEPRECATED_POP
 	/** \brief returns all packages in the cache whose name matchs a given pattern
 
 	    A simple helper responsible for executing a regular expression on all
-	    package names in the cache. Optional it prints a a notice about the
+	    package names in the cache. Optional it prints a notice about the
 	    packages chosen cause of the given package.
 	    \param Cache the packages are in
 	    \param pattern regular expression for package names
