@@ -46,7 +46,6 @@ class FileFd
    friend class Bz2FileFdPrivate;
    friend class LzmaFileFdPrivate;
    friend class Lz4FileFdPrivate;
-   friend class ZstdFileFdPrivate;
    friend class DirectFileFdPrivate;
    friend class PipedFileFdPrivate;
    protected:
@@ -77,19 +76,8 @@ class FileFd
 	ReadOnlyGzip,
 	WriteAtomic = ReadWrite | Create | Atomic
    };
-   enum CompressMode
-   {
-      Auto = 'A',
-      None = 'N',
-      Extension = 'E',
-      Gzip = 'G',
-      Bzip2 = 'B',
-      Lzma = 'L',
-      Xz = 'X',
-      Lz4 = '4',
-      Zstd = 'Z'
-   };
-
+   enum CompressMode { Auto = 'A', None = 'N', Extension = 'E', Gzip = 'G', Bzip2 = 'B', Lzma = 'L', Xz = 'X', Lz4='4' };
+   
    inline bool Read(void *To,unsigned long long Size,bool AllowEof)
    {
       unsigned long long Jnk;
@@ -98,6 +86,7 @@ class FileFd
       return Read(To,Size);
    }   
    bool Read(void *To,unsigned long long Size,unsigned long long *Actual = 0);
+   bool static Read(int const Fd, void *To, unsigned long long Size, unsigned long long * const Actual = 0);
    char* ReadLine(char *To, unsigned long long const Size);
    bool Flush();
    bool Write(const void *From,unsigned long long Size);
@@ -246,6 +235,8 @@ std::string flCombine(std::string Dir,std::string File);
 /** \brief Takes a file path and returns the absolute path
  */
 std::string flAbsPath(std::string File);
+/** \brief removes superfluous /./ and // from path */
+APT_HIDDEN std::string flNormalize(std::string file);
 
 // simple c++ glob
 std::vector<std::string> Glob(std::string const &pattern, int flags=0);
@@ -265,6 +256,4 @@ bool Popen(const char* Args[], FileFd &Fd, pid_t &Child, FileFd::OpenMode Mode,
 bool Popen(const char* Args[], FileFd &Fd, pid_t &Child, FileFd::OpenMode Mode);
 
 
-APT_HIDDEN int Inhibit(const char *what, const char *who, const char *why, const char *mode);
-
 #endif
