@@ -15,10 +15,16 @@
 #include <apt-pkg/error.h>
 #include <apt-pkg/pkgsystem.h>
 #include <apt-pkg/configuration.h>
+#include <apt-pkg/strutl.h>
 #include <apt-pkg/macros.h>
 
 #include <string.h>
 #include <cstdlib>
+#include <fstream>
+#include <sstream>
+#include <string>
+#include <unordered_map>
+#include <vector>
 
 #include <apti18n.h>
 									/*}}}*/
@@ -29,6 +35,89 @@ const char *pkgVersion = PACKAGE_VERSION;
 const char *pkgLibVersion = Stringfy(APT_PKG_MAJOR) "."
                             Stringfy(APT_PKG_MINOR) "." 
                             Stringfy(APT_PKG_RELEASE);
+namespace APT {
+   APT_HIDDEN extern std::unordered_map<std::string, std::vector<std::string>> ArchToTupleMap;
+}
+
+// Splits by whitespace. There may be continuous spans of whitespace - they
+// will be considered as one.
+static std::vector<std::string> split(std::string const & s)
+{
+   std::vector<std::string> vec;
+   std::istringstream iss(s);
+   iss.imbue(std::locale::classic());
+   for(std::string current_s; iss >> current_s; )
+      vec.push_back(current_s);
+   return vec;
+}
+
+
+// pkgInitArchTupleMap - Initialize the architecture tuple map				/*{{{*/
+// ---------------------------------------------------------------------
+/* This initializes */
+static bool pkgInitArchTupleMap()
+{
+   auto tuplepath = _config->FindFile("Dir::dpkg::tupletable", DPKG_DATADIR "/tupletable");
+   auto tripletpath = _config->FindFile("Dir::dpkg::triplettable", DPKG_DATADIR "/triplettable");
+   auto cpupath = _config->FindFile("Dir::dpkg::cputable", DPKG_DATADIR "/cputable");
+
+   // Load a list of CPUs
+   std::vector<std::string> cpus;
+   std::ifstream cputable(cpupath);
+   for (std::string cpuline; std::getline(cputable, cpuline); )
+   {
+      if (cpuline[0] == '#' || cpuline[0] == '\0')
+         continue;
+      auto cpurow = split(cpuline);
+      auto cpu = APT::String::Strip(cpurow.at(0));
+
+      cpus.push_back(cpu);
+   }
+   if (!cputable.eof())
+      return _error->Error("Error reading the CPU table");
+
+   // Load the architecture tuple
+   std::ifstream tupletable;
+   if (FileExists(tuplepath))
+      tupletable.open(tuplepath);
+   else if (FileExists(tripletpath))
+      tupletable.open(tripletpath);
+   else
+      return _error->Error("Cannot find dpkg tuplet or triplet table");
+
+   APT::ArchToTupleMap.clear();
+   for (std::string tupleline; std::getline(tupletable, tupleline); )
+   {
+      if (tupleline[0] == '#' || tupleline[0] == '\0')
+         continue;
+
+      std::vector<std::string> tuplerow = split(tupleline);
+
+      auto tuple = APT::String::Strip(tuplerow.at(0));
+      auto arch = APT::String::Strip(tuplerow.at(1));
+
+      if (tuple.find("<cpu>") == tuple.npos && arch.find("<cpu>") == arch.npos)
+      {
+         APT::ArchToTupleMap.insert({arch, VectorizeString(tuple, '-')});
+      }
+      else
+      {
+         for (auto && cpu : cpus)
+         {
+            auto mytuple = SubstVar(tuple, std::string("<cpu>"), cpu);
+            auto myarch = SubstVar(arch, std::string("<cpu>"), cpu);
+
+            APT::ArchToTupleMap.insert({myarch, VectorizeString(mytuple, '-')});
+         }
+      }
+   }
+   if (!tupletable.eof())
+      return _error->Error("Error reading the Tuple table");
+
+   return true;
+}
+									/*}}}*/
+
     
 // pkgInitConfig - Initialize the configuration class			/*{{{*/
 // ---------------------------------------------------------------------
@@ -46,37 +135,38 @@ bool pkgInitConfig(Configuration &Cnf)
    Cnf.CndSet("Dir","/");
    
    // State
-   Cnf.CndSet("Dir::State","var/lib/apt/");
+   Cnf.CndSet("Dir::State", STATE_DIR + 1);
    Cnf.CndSet("Dir::State::lists","lists/");
    Cnf.CndSet("Dir::State::cdroms","cdroms.list");
    Cnf.CndSet("Dir::State::mirrors","mirrors/");
 
    // Cache
-   Cnf.CndSet("Dir::Cache","var/cache/apt/");
+   Cnf.CndSet("Dir::Cache", CACHE_DIR + 1);
    Cnf.CndSet("Dir::Cache::archives","archives/");
    Cnf.CndSet("Dir::Cache::srcpkgcache","srcpkgcache.bin");
    Cnf.CndSet("Dir::Cache::pkgcache","pkgcache.bin");
-   
+
    // Configuration
-   Cnf.CndSet("Dir::Etc","etc/apt/");
+   Cnf.CndSet("Dir::Etc", CONF_DIR + 1);
    Cnf.CndSet("Dir::Etc::sourcelist","sources.list");
    Cnf.CndSet("Dir::Etc::sourceparts","sources.list.d");
    Cnf.CndSet("Dir::Etc::main","apt.conf");
    Cnf.CndSet("Dir::Etc::netrc", "auth.conf");
-   Cnf.CndSet("Dir::Etc::netrcparts", "auth.conf.d");
    Cnf.CndSet("Dir::Etc::parts","apt.conf.d");
    Cnf.CndSet("Dir::Etc::preferences","preferences");
    Cnf.CndSet("Dir::Etc::preferencesparts","preferences.d");
    Cnf.CndSet("Dir::Etc::trusted", "trusted.gpg");
    Cnf.CndSet("Dir::Etc::trustedparts","trusted.gpg.d");
-   Cnf.CndSet("Dir::Bin::methods","/usr/lib/apt/methods");
-   Cnf.CndSet("Dir::Bin::solvers::","/usr/lib/apt/solvers");
+   Cnf.CndSet("Dir::Bin::methods", LIBEXEC_DIR "/methods");
+   Cnf.CndSet("Dir::Bin::solvers::",LIBEXEC_DIR  "/solvers");
+   Cnf.CndSet("Dir::Bin::planners::",LIBEXEC_DIR  "/planners");
    Cnf.CndSet("Dir::Media::MountPath","/media/apt");
 
-   // State   
-   Cnf.CndSet("Dir::Log","var/log/apt");
+   // State
+   Cnf.CndSet("Dir::Log", LOG_DIR + 1);
    Cnf.CndSet("Dir::Log::Terminal","term.log");
    Cnf.CndSet("Dir::Log::History","history.log");
+   Cnf.CndSet("Dir::Log::Planner","eipp.log.xz");
 
    Cnf.Set("Dir::Ignore-Files-Silently::", "~$");
    Cnf.Set("Dir::Ignore-Files-Silently::", "\\.disabled$");
@@ -88,10 +178,8 @@ bool pkgInitConfig(Configuration &Cnf)
    Cnf.Set("Dir::Ignore-Files-Silently::", "\\.distUpgrade$");
 
    // Repository security
-   // FIXME: this is set to "true" for backward compatibility, once
-   //        jessie is out we want to change this to "false" to
-   //        improve security
-   Cnf.CndSet("Acquire::AllowInsecureRepositories", true);
+   Cnf.CndSet("Acquire::AllowInsecureRepositories", false);
+   Cnf.CndSet("Acquire::AllowWeakRepositories", false);
    Cnf.CndSet("Acquire::AllowDowngradeToInsecureRepositories", false);
 
    // Default cdrom mount point
@@ -137,14 +225,14 @@ bool pkgInitConfig(Configuration &Cnf)
    }
 
    // Read the configuration parts dir
-   std::string Parts = Cnf.FindDir("Dir::Etc::parts");
+   std::string const Parts = Cnf.FindDir("Dir::Etc::parts", "/dev/null");
    if (DirectoryExists(Parts) == true)
       Res &= ReadConfigDir(Cnf,Parts);
-   else
+   else if (APT::String::Endswith(Parts, "/dev/null") == false)
       _error->WarningE("DirectoryExists",_("Unable to read %s"),Parts.c_str());
 
    // Read the main config file
-   std::string FName = Cnf.FindFile("Dir::Etc::main");
+   std::string const FName = Cnf.FindFile("Dir::Etc::main", "/dev/null");
    if (RealFileExists(FName) == true)
       Res &= ReadConfigFile(Cnf,FName);
 
@@ -194,6 +282,9 @@ bool pkgInitSystem(Configuration &Cnf,pkgSystem *&Sys)
       if (Sys == 0)
 	 return _error->Error(_("Unable to determine a suitable packaging system type"));
    }
+
+   if (pkgInitArchTupleMap() == false)
+      return false;
    
    return Sys->Initialize(Cnf);
 }
