@@ -49,10 +49,22 @@ class pkgTagFilePrivate;
 class pkgTagSection
 {
    const char *Section;
-   unsigned int AlphaIndexes[0x100];
+   unsigned int AlphaIndexes[128];
+   unsigned int BetaIndexes[128];
 
    pkgTagSectionPrivate * const d;
 
+   APT_HIDDEN bool FindInternal(unsigned int Pos,const char *&Start, const char *&End) const;
+#if defined(APT_PKG_EXPOSE_STRING_VIEW)
+   APT_HIDDEN APT::StringView FindInternal(unsigned int Pos) const;
+   APT_HIDDEN APT::StringView FindRawInternal(unsigned int Pos) const;
+#endif
+   APT_HIDDEN signed int FindIInternal(unsigned int Pos,signed long Default = 0) const;
+   APT_HIDDEN bool FindBInternal(unsigned int Pos, bool Default = false) const;
+   APT_HIDDEN unsigned long long FindULLInternal(unsigned int Pos, unsigned long long const &Default = 0) const;
+   APT_HIDDEN bool FindFlagInternal(unsigned int Pos,uint8_t &Flags, uint8_t const Flag) const;
+   APT_HIDDEN bool FindFlagInternal(unsigned int Pos,unsigned long &Flags, unsigned long Flag) const;
+
    protected:
    const char *Stop;
 
@@ -77,7 +89,19 @@ class pkgTagSection
    std::string FindS(const char *Tag) const;
    std::string FindRawS(const char *Tag) const;
 
+   // Functions for lookup with a perfect hash function
+   enum class Key;
+   APT_HIDDEN bool Find(Key key,const char *&Start, const char *&End) const;
+   APT_HIDDEN bool Find(Key key,unsigned int &Pos) const;
+   APT_HIDDEN signed int FindI(Key key,signed long Default = 0) const;
+   APT_HIDDEN bool FindB(Key key, bool Default = false) const;
+   APT_HIDDEN unsigned long long FindULL(Key key, unsigned long long const &Default = 0) const;
+   APT_HIDDEN bool FindFlag(Key key,uint8_t &Flags, uint8_t const Flag) const;
+   APT_HIDDEN bool FindFlag(Key key,unsigned long &Flags, unsigned long Flag) const;
+   APT_HIDDEN bool Exists(Key key) const;
 #ifdef APT_PKG_EXPOSE_STRING_VIEW
+   APT_HIDDEN APT::StringView Find(Key key) const;
+   APT_HIDDEN APT::StringView FindRaw(Key key) const;
    APT_HIDDEN bool Find(APT::StringView Tag,const char *&Start, const char *&End) const;
    APT_HIDDEN bool Find(APT::StringView Tag,unsigned int &Pos) const;
    APT_HIDDEN APT::StringView Find(APT::StringView Tag) const;
@@ -122,7 +146,7 @@ class pkgTagSection
 
    /** \brief amount of Tags in the current section
     *
-    * Note: if a Tag is mentioned repeatly it will be counted multiple
+    * Note: if a Tag is mentioned repeatedly it will be counted multiple
     * times, but only the last occurrence is available via Find methods.
     */
    unsigned int Count() const;
