@@ -24,6 +24,7 @@
 #include <apt-private/private-show.h>
 
 #include <stdio.h>
+#include <unistd.h>
 #include <ostream>
 #include <string>
 
@@ -177,9 +178,9 @@ static bool DisplayRecordV2(pkgCacheFile &CacheFile, pkgCache::VerIterator const
 
    // Read the record
    pkgTagSection Tags;
-   pkgTagFile TagF(&PkgF, Vf->Size);
+   pkgTagFile TagF(&PkgF);
 
-   if (TagF.Jump(Tags, Vf->Offset) == false)
+   if (TagF.Jump(Tags, V.FileList()->Offset) == false)
       return _error->Error("Internal Error, Unable to parse a package record");
 
    // make size nice
@@ -368,8 +369,6 @@ bool Policy(CommandLine &CmdL)
    if (unlikely(Plcy == nullptr))
       return false;
 
-   bool OldPolicy = _config->FindI("APT::Policy", 1) < 1;
-
    // Print out all of the package files
    if (CmdL.FileList[1] == 0)
    {
@@ -400,24 +399,6 @@ bool Policy(CommandLine &CmdL)
       pkgCache::PkgIterator I = Cache->PkgBegin();
       for (;I.end() != true; ++I)
       {
-	 // Old code for debugging
-	 if (OldPolicy)
-	 {
-	    if (Plcy->GetPriority(I) == 0)
-	       continue;
-
-	    // Print the package name and the version we are forcing to
-	    std::cout << "     " << I.FullName(true) << " -> ";
-
-	    pkgCache::VerIterator V = Plcy->GetMatch(I);
-	    if (V.end() == true)
-	       std::cout << _("(not found)") << std::endl;
-	    else
-	       std::cout << V.VerStr() << std::endl;
-
-	    continue;
-	 }
-	 // New code
 	 for (pkgCache::VerIterator V = I.VersionList(); !V.end(); ++V) {
 	    auto Prio = Plcy->GetPriority(V, false);
 	    if (Prio == 0)
@@ -462,17 +443,6 @@ bool Policy(CommandLine &CmdL)
       else
 	 std::cout << V.VerStr() << std::endl;
 
-      // Pinned version
-      if (OldPolicy && Plcy->GetPriority(Pkg) != 0)
-      {
-	 std::cout << _("  Package pin: ");
-	 V = Plcy->GetMatch(Pkg);
-	 if (V.end() == true)
-	    std::cout << _("(not found)") << std::endl;
-	 else
-	    std::cout << V.VerStr() << std::endl;
-      }
-
       // Show the priority tables
       std::cout << _("  Version table:") << std::endl;
       for (V = Pkg.VersionList(); V.end() == false; ++V)
@@ -481,10 +451,8 @@ bool Policy(CommandLine &CmdL)
 	    std::cout << " *** " << V.VerStr();
 	 else
 	    std::cout << "     " << V.VerStr();
-	 if (_config->FindI("APT::Policy", 1) < 1)
-	    std::cout << " " << Plcy->GetPriority(Pkg) << std::endl;
-	 else
-	    std::cout << " " << Plcy->GetPriority(V) << std::endl;
+
+	 std::cout << " " << Plcy->GetPriority(V) << std::endl;
 	 for (pkgCache::VerFileIterator VF = V.FileList(); VF.end() == false; ++VF)
 	 {
 	    // Locate the associated index files so we can derive a description
