@@ -2,14 +2,11 @@
 #include <config.h>
 
 #include <apt-pkg/upgrade.h>
-#include <apt-pkg/cmndline.h>
 #include <apt-pkg/configuration.h>
 #include <apt-pkg/error.h>
 
 #include <apt-private/private-install.h>
 #include <apt-private/private-cachefile.h>
-#include <apt-private/private-json-hooks.h>
-#include <apt-private/private-output.h>
 #include <apt-private/private-upgrade.h>
 #include <apt-private/private-output.h>
 
@@ -22,24 +19,16 @@
 static bool UpgradeHelper(CommandLine &CmdL, int UpgradeFlags)
 {
    CacheFile Cache;
-   std::vector<char const *> VolatileCmdL;
+   std::vector<std::string> VolatileCmdL;
    Cache.GetSourceList()->AddVolatileFiles(CmdL, &VolatileCmdL);
 
    if (Cache.OpenForInstall() == false || Cache.CheckDeps() == false)
       return false;
 
-   std::map<unsigned short, APT::VersionSet> verset;
-   std::set<std::string> UnknownPackages;
-   if (!DoCacheManipulationFromCommandLine(CmdL, VolatileCmdL, Cache, verset, UpgradeFlags, UnknownPackages))
-   {
-      RunJsonHook("AptCli::Hooks::Upgrade", "org.debian.apt.hooks.install.fail", CmdL.FileList, Cache, UnknownPackages);
+   if(!DoCacheManipulationFromCommandLine(CmdL, VolatileCmdL,  Cache, UpgradeFlags))
       return false;
-   }
-   RunJsonHook("AptCli::Hooks::Upgrade", "org.debian.apt.hooks.install.pre-prompt", CmdL.FileList, Cache);
-   if (InstallPackages(Cache, true, true, true, "AptCli::Hooks::Upgrade", CmdL))
-      return RunJsonHook("AptCli::Hooks::Upgrade", "org.debian.apt.hooks.install.post", CmdL.FileList, Cache);
-   else
-      return RunJsonHook("AptCli::Hooks::Upgrade", "org.debian.apt.hooks.install.fail", CmdL.FileList, Cache);
+
+   return InstallPackages(Cache,true);
 }
 
 // DoDistUpgrade - Automatic smart upgrader				/*{{{*/
