@@ -6,7 +6,7 @@
    FTP Acquire Method - This is the FTP acquire method for APT.
 
    This is a very simple implementation that does not try to optimize
-   at all. Commands are sent syncronously with the FTP server (as the
+   at all. Commands are sent synchronously with the FTP server (as the
    rfc recommends, but it is not really necessary..) and no tricks are
    done to speed things along.
 			
@@ -18,7 +18,6 @@
 #include <config.h>
 
 #include <apt-pkg/fileutl.h>
-#include <apt-pkg/acquire-method.h>
 #include <apt-pkg/error.h>
 #include <apt-pkg/hashes.h>
 #include <apt-pkg/netrc.h>
@@ -975,7 +974,7 @@ FtpMethod::FtpMethod() : aptMethod("ftp","1.0",SendConfig)
 // FtpMethod::SigTerm - Handle a fatal signal				/*{{{*/
 // ---------------------------------------------------------------------
 /* This closes and timestamps the open file. This is necessary to get
-   resume behavoir on user abort */
+   resume behavior on user abort */
 void FtpMethod::SigTerm(int)
 {
    if (FailFd == -1)
@@ -1018,22 +1017,6 @@ bool FtpMethod::Fetch(FetchItem *Itm)
    Res.IMSHit = false;
 
    maybe_add_auth (Get, _config->FindFile("Dir::Etc::netrc"));
-   if(Get.User.empty() || Get.Password.empty())
-   {
-      auto const netrcparts = _config->FindDir("Dir::Etc::netrcparts");
-      if (not netrcparts.empty())
-      {
-	 _error->PushToStack();
-	 for (auto const &netrc : GetListOfFilesInDir(netrcparts, "conf", true, true))
-	 {
-	    maybe_add_auth (Get, netrc);
-	    if (Get.User.empty() == false || Get.Password.empty() == false)
-	       break;
-	 }
-	 _error->RevertToStack();
-      }
-   }
-
 
    // Connect to the server
    if (Server == 0 || Server->Comp(Get) == false)
@@ -1144,9 +1127,7 @@ bool FtpMethod::Fetch(FetchItem *Itm)
 									/*}}}*/
 
 int main(int, const char *argv[])
-{ 
-   setlocale(LC_ALL, "");
-
+{
    /* See if we should be come the http client - we do this for http
       proxy urls */
    if (getenv("ftp_proxy") != 0)
@@ -1169,8 +1150,5 @@ int main(int, const char *argv[])
 	 exit(100);
       }      
    }
-   
-   FtpMethod Mth;
-
-   return Mth.Run();
+   return FtpMethod().Run();
 }
