@@ -10,7 +10,6 @@
 #include <apt-pkg/init.h>
 #include <apt-pkg/fileutl.h>
 #include <apt-pkg/error.h>
-#include <apt-pkg/acquire-method.h>
 #include <apt-pkg/strutl.h>
 #include <apt-pkg/hashes.h>
 #include <apt-pkg/configuration.h>
@@ -574,7 +573,7 @@ class RredMethod : public aptMethod {
 
    protected:
       virtual bool URIAcquire(std::string const &Message, FetchItem *Itm) APT_OVERRIDE {
-	 Debug = _config->FindB("Debug::pkgAcquire::RRed", false);
+	 Debug = DebugEnabled();
 	 URI Get = Itm->Uri;
 	 std::string Path = Get.Host + Get.Path; // rred:/path - no host
 
@@ -663,12 +662,14 @@ class RredMethod : public aptMethod {
 	 FileFd inp, out;
 	 if (inp.Open(Path, FileFd::ReadOnly, FileFd::Extension) == false)
 	 {
-	    std::cerr << "FAILED to open inp " << Path << std::endl;
+	    if (Debug == true)
+	       std::clog << "FAILED to open inp " << Path << std::endl;
 	    return _error->Error("Failed to open inp %s", Path.c_str());
 	 }
 	 if (out.Open(Itm->DestFile, FileFd::WriteOnly | FileFd::Create | FileFd::Empty | FileFd::BufferedWrite, FileFd::Extension) == false)
 	 {
-	    std::cerr << "FAILED to open out " << Itm->DestFile << std::endl;
+	    if (Debug == true)
+	       std::clog << "FAILED to open out " << Itm->DestFile << std::endl;
 	    return _error->Error("Failed to open out %s", Itm->DestFile.c_str());
 	 }
 
@@ -687,7 +688,8 @@ class RredMethod : public aptMethod {
 	 inp.Close();
 
 	 if (_error->PendingError() == true) {
-	    std::cerr << "FAILED to read or write files" << std::endl;
+	    if (Debug == true)
+	       std::clog << "FAILED to read or write files" << std::endl;
 	    return false;
 	 }
 
@@ -730,8 +732,7 @@ int main(int argc, char **argv)
    Patch patch;
 
    if (argc <= 1) {
-      RredMethod Mth;
-      return Mth.Run();
+      return RredMethod().Run();
    }
 
    // Usage: rred -t input output diff ...
